@@ -1,4 +1,5 @@
 """Main module."""
+import csv
 import functools
 import shutil
 import sys
@@ -11,6 +12,8 @@ from pathlib import Path
 import psutil
 from typing import Dict, Tuple, Optional
 
+import pytimings
+
 from pytimings.mpi import get_communication_wrapper, get_local_communicator, get_communicator
 from pytimings.tools import ensure_directory_exists
 
@@ -32,7 +35,7 @@ WALL_TIME = "wall"
 SYS_TIME = "sys"
 USER_TIME = "user"
 
-TimingDelta = namedtuple("TimingDelta", [WALL_TIME, SYS_TIME, USER_TIME], defaults=[0, 0, 0])
+TimingDelta = namedtuple("TimingDelta", [WALL_TIME, SYS_TIME, USER_TIME])
 
 
 class NoTimerError(Exception):
@@ -83,7 +86,7 @@ class Timings:
     def __init__(self):
         self._commited_deltas: Dict[str, TimingDelta] = {}
         self._known_timers_map: Dict[str, Tuple[bool, Optional[TimingData]]] = defaultdict(_default_timer_dict_entry)
-        self._csv_sep = ","
+        self.extra_data = dict()
         self.reset()
 
     def start(self, section_name: str) -> None:
@@ -128,14 +131,14 @@ class Timings:
             self.stop(section_name)
         except NoTimerError:
             pass  # not stopping no timer is not an error
-        self._commited_deltas[section_name] = TimingDelta()
+        self._commited_deltas[section_name] = TimingDelta(0, 0, 0)
 
     def walltime(self, section_name: str) -> int:
         """get runtime of section in milliseconds"""
         return self.delta(section_name)[0]
 
     def add_walltime(self, section_name: str, time: int) -> None:
-        self._commited_deltas[section_name] = TimingDelta(time)
+        self._commited_deltas[section_name] = TimingDelta(time, 0, 0)
 
     def delta(self, section_name: str) -> Dict[str, int]:
         """get the full delta dict"""
@@ -144,18 +147,21 @@ class Timings:
         except KeyError:
             raise NoTimerError(section_name, self)
 
-    def output_per_rank(self, output_dir: Path, csv_base: str) -> None:
-        """
-             creates one file local to each MPI-rank (no global averaging)
-        *  one single rank-0 file with all combined/averaged measures
+    def output_files(self, output_dir: Path, csv_base: str, per_rank=False) -> None:
+        """creates one file local to each MPI-rank (no global averaging)
+        and one single rank-0 file with all combined/averaged measures
+
+
         """
         communication = get_communication_wrapper()
         rank = communication.rank
 
+        output_dir = Path(output_dir)
         ensure_directory_exists(output_dir)
-        filename = output_dir / f"{csv_base}_p{rank:08d}.csv"
-        with open(filename, "wt") as outfile:
-            self.output_all_measures(outfile, get_local_communicator())
+        if per_rank:
+            filename = output_dir / f"{csv_base}_p{rank:08d}.csv"
+            with open(filename, "wt") as outfile:
+                self.output_all_measures(outfile, get_local_communicator())
         tmp_out = StringIO()
         # all ranks have to participate in the data generation
         self.output_all_measures(tmp_out, get_communicator())
@@ -183,43 +189,52 @@ class Timings:
 
     def output_all_measures(self, out=None, mpi_comm=None) -> None:
         """output all recorded measures
-        * \note outputs average, min, max over all MPI processes associated to mpi_comm **/"""
+
+        Outputs average, min, max over all MPI processes associated to mpi_comm
+        """
         out = out or sys.stdout
         mpi_comm = mpi_comm or get_communicator()
         comm = get_communication_wrapper(mpi_comm)
         stash = StringIO()
-        sep = self._csv_sep
-        stash.write(f"threads{sep}ranks")
-        for section in self._commited_deltas.keys():
-            stash.write(
-                f'{sep}{section}_avg_usr{sep}{section}_max_usr{sep}{section}_avg_wall{sep}{section}_max_wall{sep}{section}_avg_sys{sep}{section}_max_sys'
-            )
+        csv_file = csv.writer(stash, lineterminator='\n')
+        # header
+        csv_file.writerow(['section', 'value'])
+
+        # threadManager().max_threads()
+        csv_file.writerow(["threads", 1])
+        csv_file.writerow(["ranks", comm.size])
 
         weight = 1 / comm.size
 
-        # threadManager().max_threads() {sep} comm.size()
-        stash.write(f"\n1{sep}{comm.size}")
-
         for section, delta in self._commited_deltas.items():
             delta = delta._asdict()
             wall = delta[WALL_TIME]
             usr = delta[USER_TIME]
             syst = delta[SYS_TIME]
-            wall_sum = comm.sum(wall)
-            wall_max = comm.max(wall)
-            usr_sum = comm.sum(usr)
-            usr_max = comm.max(usr)
-            sys_sum = comm.sum(syst)
-            sys_max = comm.max(syst)
-            stash.write(
-                f"{sep}{usr_sum * weight}{sep}{usr_max}{sep}{wall_sum * weight}{sep}{wall_max}{sep}{sys_sum * weight}{sep}{sys_max}"
+            csv_file.writerows(
+                [
+                    [f"{section}_avg_usr", comm.sum(usr) * weight],
+                    [f"{section}_max_usr", comm.max(usr)],
+                    [f"{section}_avg_wall", comm.sum(wall) * weight],
+                    [f"{section}_max_wall", comm.max(wall)],
+                    [f"{section}_avg_sys", comm.sum(syst) * weight],
+                    [f"{section}_max_sys", comm.max(syst)],
+                ]
             )
-
-        stash.write("\n")
+        csv_file.writerows([[f'pytimings::data::{k}', v] for k, v in self.extra_data.items()])
+        csv_file.writerow(['pytimings::data::_sections', '||'.join(self._commited_deltas.keys())])
+        csv_file.writerow(['pytimings::data::_version', pytimings.__version__])
         stash.seek(0)
         if comm.rank == 0:
             shutil.copyfileobj(stash, out)
 
+    def add_extra_data(self, data: [Dict]):
+        """Use this for something configuration data that makes the csv more informative.
+
+        Data is not displayed with console output.
+        """
+        self.extra_data.update(data)
+
 
 global_timings = Timings()
 
