@@ -2,32 +2,23 @@
 
 """Tests for `pytimings` package."""
 import pickle
-import sys
-import time
 import numpy as np
 from functools import partial
 from tempfile import TemporaryFile
 
 from click.testing import CliRunner
 
-from pytimings import cli, mpi
+from pytimings import cli
 from pytimings.timer import scoped_timing, function_timer
-from .fixtures import timings_object, use_mpi, pickled_timings_object, is_windows_platform
+from pytimings.tools import output_at_exit, busywait
+from .fixtures import is_windows_platform
 
 _DUMMY_SECTION = 'mysection'
 
 DEFAULT_SLEEP_SECONDS = 0.1
 
 
-def _busywait(secs):
-    """busywait simulates load, so user time won't be 0 in timings"""
-    init_time = time.time()
-    while time.time() < init_time + secs:
-        pass
-    return time.time() - init_time
-
-
-default_sleep = partial(_busywait, DEFAULT_SLEEP_SECONDS)
+default_sleep = partial(busywait, DEFAULT_SLEEP_SECONDS)
 
 
 def _assert(delta_value, lower=DEFAULT_SLEEP_SECONDS, upper=None):
@@ -122,7 +113,7 @@ def test_simple_output(timings_object):
     with scoped_timing("time", print, timings=timings_object, format='.5f'):
         default_sleep()
     with scoped_timing("a much longer section name", print, timings=timings_object):
-        _busywait(1)
+        busywait(1)
     scoped_sleepy_time = timings_object.walltime("time")
     timings_object.output_console()
 
@@ -132,6 +123,11 @@ def test_simple_output(timings_object):
     np.isclose(scoped_sleepy_time, sleepy_time)
 
 
+def test_atexit(timings_object):
+    """This only tests if the setup function can be called"""
+    output_at_exit(timings=timings_object)
+
+
 def test_command_line_interface():
     """Test the CLI."""
     runner = CliRunner()
